@@ -22,9 +22,11 @@ gi.require_version('Gtk', '3.0')
 gi.require_version('Gdk', '3.0')
 from gi.repository import GLib, Gio
 from gi.repository import Gtk, Gdk, GdkPixbuf
-import cgi, os
+import cgi
+import os
 import gettext
 import subprocess
+import shutil
 import urllib
 try:
 	from Mozo import config
@@ -295,6 +297,9 @@ class MainWindow:
 			return False
 		return True
 
+	def on_delete_event(self, widget, event):
+		self.quit()
+
 	def on_new_menu_button_clicked(self, button):
 		menu_tree = self.tree.get_object('menu_tree')
 		menus, iter = menu_tree.get_selection().get_selected()
@@ -382,8 +387,7 @@ class MainWindow:
 			file_type = 'Menu'
 
 		if not os.path.isfile(file_path):
-			data = open(item.get_desktop_file_path()).read()
-			open(file_path, 'w').write(data)
+			shutil.copy(item.get_desktop_file_path(), file_path)
 			self.editor._MenuEditor__addUndo([(file_type, os.path.split(file_path)[1]),])
 		else:
 			self.editor._MenuEditor__addUndo([item,])
