@@ -16,7 +16,12 @@
 #   License along with this library; if not, write to the Free Software
 #   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-import os, re, xml.dom.minidom, locale
+import codecs
+import os
+import re
+import xml.dom.minidom
+import xml.parsers.expat
+import locale
 import matemenu
 from Mozo import util
 
@@ -42,10 +47,10 @@ class MenuEditor:
 		self.applications.tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.applications.visible_tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.applications.path = os.path.join(util.getUserMenuPath(), self.applications.tree.get_menu_file())
-		if not os.path.isfile(self.applications.path):
-			self.applications.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.applications.tree))
-		else:
+		try:
 			self.applications.dom = xml.dom.minidom.parse(self.applications.path)
+		except (IOError, xml.parsers.expat.ExpatError):
+			self.applications.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.applications.tree))
 		self.__remove_whilespace_nodes(self.applications.dom)
 
 		self.settings = Menu()
@@ -54,19 +59,19 @@ class MenuEditor:
 		self.settings.tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.settings.visible_tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.settings.path = os.path.join(util.getUserMenuPath(), self.settings.tree.get_menu_file())
-		if not os.path.isfile(self.settings.path):
-			self.settings.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.settings.tree))
-		else:
+		try:
 			self.settings.dom = xml.dom.minidom.parse(self.settings.path)
+		except (IOError, xml.parsers.expat.ExpatError):
+			self.settings.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.settings.tree))
 		self.__remove_whilespace_nodes(self.settings.dom)
 
 		self.save(True)
 
 	def save(self, from_loading=False):
 		for menu in ('applications', 'settings'):
-			fd = open(getattr(self, menu).path, 'w')
-			fd.write(re.sub("\n[\s]*([^\n<]*)\n[\s]*</", "\\1</", getattr(self, menu).dom.toprettyxml().replace('<?xml version="1.0" ?>\n', '')))
-			fd.close()
+			xml = getattr(self, menu).dom.toprettyxml().replace('<?xml version="1.0" ?>\n', '')
+			with codecs.open(getattr(self, menu).path, 'w', 'utf-8') as f:
+				f.write(re.sub("\n[\s]*([^\n<]*)\n[\s]*</", "\\1</", xml))
 		if not from_loading:
 			self.__loadMenus()
 
@@ -94,10 +99,10 @@ class MenuEditor:
 			except OSError:
 				pass
 			#reload DOM for each menu
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
+			try:
 				menu.dom = xml.dom.minidom.parse(menu.path)
+			except (IOError, xml.parsers.expat.ExpatError):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
 			self.__remove_whilespace_nodes(menu.dom)
 		#reset undo/redo, no way to recover from this
 		self.__undo, self.__redo = [], []
@@ -119,19 +124,29 @@ class MenuEditor:
 		for file_path in files:
 			new_path = file_path.rsplit('.', 1)[0]
 			redo_path = util.getUniqueRedoFile(new_path)
-			data = open(new_path).read()
-			open(redo_path, 'w').write(data)
-			data = open(file_path).read()
-			open(new_path, 'w').write(data)
+
+			f_file_path = codecs.open(new_path, 'r', 'utf-8')
+			f_new_path = codecs.open(new_path, 'rw', 'utf-8')
+			f_redo_path = codecs.open(redo_path, 'rw', 'utf-8')
+
+			data = f_new_path.read()
+			f_redo_path.write(data)
+			data = f_file_path.read()
+			f_new_path.write(data)
+
+			f_file_path.close()
+			f_new_path.close()
+			f_redo_path.close()
+
 			os.unlink(file_path)
 			redo.append(redo_path)
-		#reload DOM to make changes stick
+		# reload DOM to make changes stick
 		for name in ('applications', 'settings'):
 			menu = getattr(self, name)
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
+			try:
 				menu.dom = xml.dom.minidom.parse(menu.path)
+			except (IOError, xml.parsers.expat.ExpatError):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
 			self.__remove_whilespace_nodes(menu.dom)
 		self.__redo.append(redo)
 
@@ -143,19 +158,25 @@ class MenuEditor:
 		for file_path in files:
 			new_path = file_path.rsplit('.', 1)[0]
 			undo_path = util.getUniqueUndoFile(new_path)
-			data = open(new_path).read()
-			open(undo_path, 'w').write(data)
-			data = open(file_path).read()
-			open(new_path, 'w').write(data)
+			f_file_path = codecs.open(new_path, 'r', 'utf-8')
+			f_new_path = codecs.open(new_path, 'rw', 'utf-8')
+			f_undo_path = codecs.open(undo_path, 'rw', 'utf-8')
+			data = f_new_path.read()
+			f_undo_path.write(data)
+			data = f_file_path.read()
+			f_new_path.write(data)
 			os.unlink(file_path)
 			undo.append(undo_path)
+			f_file_path.close()
+			f_new_path.close()
+			f_undo_path.close()
 		#reload DOM to make changes stick
 		for name in ('applications', 'settings'):
 			menu = getattr(self, name)
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
+			try:
 				menu.dom = xml.dom.minidom.parse(menu.path)
+			except (IOError, xml.parsers.expat.ExpatError):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
 			self.__remove_whilespace_nodes(menu.dom)
 		self.__undo.append(undo)
 
@@ -424,9 +445,11 @@ class MenuEditor:
 					file_path = item.get_desktop_file_path()
 			else:
 				continue
-			data = open(file_path).read()
+			with codecs.open(file_path, 'r', 'utf-8') as f:
+				data = f.read()
 			undo_path = util.getUniqueUndoFile(file_path)
-			open(undo_path, 'w').write(data)
+			with codecs.open(undo_path, 'w', 'utf-8') as f:
+				f.write(data)
 			self.__undo[-1].append(undo_path)
 
 	def __getMenu(self, item):
@@ -478,7 +501,7 @@ class MenuEditor:
 
 	def __getPath(self, menu, path=None):
 		if not path:
-                        path = menu.tree.root.get_menu_id()
+			path = menu.tree.root.get_menu_id()
 		if menu.get_parent():
 			path = self.__getPath(menu.get_parent(), path)
 			path += '/'
@@ -513,11 +536,19 @@ class MenuEditor:
 		return found
 
 	def __addXmlMenuElement(self, element, name, dom):
+		if isinstance(name, bytes):
+			name = name.decode('utf-8')
+
 		node = dom.createElement('Menu')
 		self.__addXmlTextElement(node, 'Name', name, dom)
 		return element.appendChild(node)
 
 	def __addXmlTextElement(self, element, name, text, dom):
+		if isinstance(name, bytes):
+			name = name.decode('utf-8')
+		if isinstance(text, bytes):
+			text = text.decode('utf-8')
+
 		for temp in element.childNodes:
 			if temp.nodeName == name:
 				if temp.childNodes[0].nodeValue == text:
@@ -528,6 +559,9 @@ class MenuEditor:
 		return element.appendChild(node)
 
 	def __addXmlFilename(self, element, dom, filename, type = 'Include'):
+		if isinstance(filename, bytes):
+			filename = filename.decode('utf-8')
+
 		# remove old filenames
 		for node in self.__getXmlNodesByName(['Include', 'Exclude'], element):
 			if node.childNodes[0].nodeName == 'Filename' and node.childNodes[0].childNodes[0].nodeValue == filename:
